(*************************************************
pvlListCombo is a Dropdown combobox that can
automatically update and INI file, The registry
or flat file with new entries.

New Properties
RegistryKey : The registry path after currentuser to update. (32 bit only)
AutoSave : If True the component will save the list whenever it has changed.
PromptBeforeAdd : If True the user will be prompted before adding the item.
LoadInDesigner : If True The Item List will be updated in design mode.
Prompt : The Prompt to display to the user. It gets passed to format with new item
         as a single string parameter.
Section : The Section of the INI file/Registry where the items get saved.
IniFile : The Name of the INI file.
FlatFile : The Name of a flat file where the items get saved.

New Events
OnItemsChanged : This gets triggered whenever an item is added.

New Methods
SaveItems : This is to manually save the items.

NOTE : If Anyone has a nice DCR file for this component.
       Please forward it as I've run out of ideas. ;-)

***************************************************)
(*************************************************
This Source Code remains Copyrighted to
CopyRight (c) 1998 PA van Lonkhuyzen
It may freely distributed and used as long as this message
Isn't removed.

The Author may be contacted via
email peterv@global.co.za
post P A van Lonkhuyzen
     PO Box 1041
     Randpark Ridge
     2156
     South Africa
***************************************************)
{$D-,L-} {Remove if you wish to debug this code}
unit pvlLstCb;

interface
uses
{$ifdef win32}
  Windows,
{$endif}
  Messages, SysUtils, Classes,  Controls,  Dialogs,  StdCtrls,Graphics,menus;

type
  TpvlListCombo = class(TCustomComboBox)
  private
     fAutoSave,
     fPromptbeforeSave,
     fLoadinDesigner : Boolean;
     fIniFile,
     fSection,
     fFile,
     fPrompt
{$IFDEF win32}
     ,fRegistryKey
{$endif}: String;
     fOnItemsChanged : TNotifyEvent;
  protected
     procedure DoExit; Override;
     Procedure SetSection(const AString : String);
     Procedure SetIniFile(const AString : String);
     Procedure SetFile(const AString : String);
{$IFDEF WIN32}
     Procedure SetRegistryKey(const AString : String);
{$ENDIF}
    Procedure Loaded; override;
  public
    constructor Create(AOwner: TComponent); override;
    destructor Destroy; override;
    Procedure SaveItems;
  published
{$IFDEF win32}
    Property RegistryKey : String Read fRegistryKey Write SetRegistryKey;
{$ENDIF}
    Property AutoSave : Boolean read fAutoSave write fAutoSave default true;
    Property PromptBeforeAdd : Boolean read fPromptBeforeSave write fPromptBeforeSave;
    Property LoadInDesigner : Boolean read fLoadInDesigner write fLoadInDesigner;
    Property Prompt : String read fPrompt write fPrompt;
    Property OnItemsChanged : TNotifyevent read fOnItemsChanged write fOnItemsChanged;
    Property Section : String Read fSection Write SetSection;
    Property IniFile : String Read fIniFile Write SetIniFile;
    Property FlatFile : String Read fFile Write SetFile;
    property Color;
    property Ctl3D;
    property DragMode;
    property DragCursor;
    property DropDownCount;
    property Enabled;
    property Font;
{$ifndef ver80}
{$ifndef ver90}
    property ImeMode;
    property ImeName;
{$endif}
    property OnStartDrag;
{$endif}
    property ItemHeight;
    property Items;
    property MaxLength;
    property ParentColor;
    property ParentCtl3D;
    property ParentFont;
    property ParentShowHint;
    property PopupMenu;
    property ShowHint;
    property Sorted;
    property TabOrder;
    property TabStop;
    property Text;
    property Visible;
    property OnChange;
    property OnClick;
    property OnDblClick;
    property OnDragDrop;
    property OnDragOver;
    property OnDropDown;
    property OnEndDrag;
    property OnEnter;
    property OnExit;
    property OnKeyDown;
    property OnKeyPress;
    property OnKeyUp;
  end;

procedure Register;

implementation
{$IfOpt I-}
  {$I+}
  {$Define PVLLCOMBOIO}
{$ENDIF}

uses inifiles{$Ifdef win32},registry{$endif};

procedure TpvlListCombo.DoExit;
begin
  inherited doexit;
  if text<>'' then
    if items.IndexOf(text)=-1 then
    begin
      if (not fpromptbeforesave) or (Messagedlg(format(prompt,[text]),mtconfirmation,[mbyes,mbno],0)=MrYes) then
      begin
        items.add(text);
        if fAutoSave then
          saveitems;
        if assigned(fOnItemsChanged) then
          fOnItemsChanged(self);
      end;
    end;
end;

Procedure TpvlListCombo.Loaded;
var s : string;
    L : TStringList;
    i : Integer;
begin
 inherited loaded;
 l:=nil;
 if (items.count<>0) and not fLoadInDesigner then
 begin
   l:=TStringlist.create;
   l.assign(items);
 end;
 try
   s:=Flatfile;
   Flatfile:='';
   Flatfile:=s;
   s:=inifile;
   inifile:='';
   inifile:=s;
  {$ifdef win32}
   s:=Registrykey;
   Registrykey:='';
   Registrykey:=s;
  {$endif}
  finally
    if l<>nil then
    begin
      try
        for i:=0 to l.count-1 do
          if items.IndexOf(l[i])=-1 then
           Items.Add(l[i]);
      except
      end;
      l.free;
    end;
  end;
end;

Procedure TpvlListCombo.SetSection(const AString : String);
var s : string;
Begin
  If (AString <> fSection) then
  begin
    fSection := AString;
    If fSection<>'' then
    begin
      s:=inifile;
      inifile:='';
      inifile:=s;
     {$ifdef win32}
      s:=Registrykey;
      Registrykey:='';
      Registrykey:=s;
     {$endif}
    end;
  end;
End;

Procedure TpvlListCombo.SetIniFile(const AString : String);
var ini : TIniFile;
Begin
  If (AString <> fIniFile) then
  begin
    fIniFile := AString;
    If fIniFile<>'' then
    begin
      Flatfile:='';
{$ifdef win32}
      registrykey:='';
{$endif}
      if (not (csloading in componentstate)) and (section<>'') and
         ((not (csdesigning in componentstate)) or fLoadinDesigner) then
      begin
        ini:=TIniFile.Create(FIniFile);
        try
          try
            Items.clear;
            ini.ReadSection(Section,Items);
          except
          end;
        finally
          ini.free
        end;
      end
    end;
  end;
End;

Procedure TpvlListCombo.SetFile(const AString : String);
Begin
  If (AString <> fFile) then
  begin
    fFile := AString;
    If fFile<>'' then
    begin
      IniFile:='';
{$ifdef win32}
      registrykey:='';
{$endif}
      if (not (csloading in componentstate)) and
         ((not (csdesigning in componentstate)) or fLoadinDesigner) then
      begin
        try
          Items.clear;
          Items.LoadFromFile(ffile);
        except
        end;
      end;
    end;
  end;
End;

{$ifdef win32}
Procedure TpvlListCombo.SetRegistryKey(const AString : String);
var ini : TRegIniFile;
Begin
  If (AString <> fRegistryKey) then
  begin
    fRegistryKey := AString;
    If fRegistryKey<>'' then
    begin
      inifile:='';
      FlatFile:='';
      if (not (csloading in componentstate)) and (section<>'') and
         ((not (csdesigning in componentstate)) or fLoadinDesigner) then
      begin
        ini:=TRegIniFile.Create(fRegistryKey);
        try
          try
            Items.clear;
            ini.ReadSection(Section,Items);
          except
          end;
        finally
          ini.free
        end;
      end
    end;
  end;
End;
{$endif}

constructor TpvlListCombo.Create(AOwner: TComponent);
begin
  Inherited create(AOwner);
  fPrompt:='%s doesn''t currently exist in the list do you wish to add it?';
  fAutoSave:=True;
  style:=csDropdown;
end;

destructor TpvlListCombo.Destroy;
begin
  Inherited Destroy;
end;

Procedure TpvlListCombo.SaveItems;
var ini : TInifile;
{$ifdef win32}
    reg : TRegIniFile;
{$Endif}
    i : Integer;
    destlist : Boolean;
begin
  if csdesigning in Componentstate then
    exit;
  if ffile<>'' then
  try
    Items.SaveToFile(ffile);
  except
  end
  else if (finifile<>'') and (fsection<>'') then
  begin
    ini:=TIniFile.Create(FIniFile);
    try
      try
        ini.EraseSection(fsection);
        if Items.count<>0 then
          for i:=0 to Items.count-1 do
            if Items[i]<>'' then
              ini.WriteString(fsection,Items[i],'');
      except
      end;
    finally
      ini.free
    end;
  end
{$Ifdef win32}
  else if (fRegistryKey<>'') and (fsection<>'') then
  begin
    Reg:=TRegIniFile.Create(fRegistryKey);
    try
      try
        Reg.EraseSection(fsection);
        if Items.count<>0 then
          for i:=0 to Items.count-1 do
            if Items[i]<>'' then
              Reg.WriteString(fsection,Items[i],'');
      except
      end;
    finally
      Reg.free
    end;
  end
{$Endif};
 if destlist then
 begin
   Items.free;
   Items:=nil;
 end;
end;

procedure Register;
begin
  RegisterComponents('Peter''s Components', [TpvlListCombo]);
end;

{$IfDef PVLLCOMBOIO}
  {$I-}
  {$UnDef PVLLCOMBOIO}
{$ENDIF}
end.
